package com.zhili.stationserver.component;

import com.alibaba.fastjson.JSON;
import com.zhili.stationserver.annotation.ProtocalTranform;
import com.zhili.stationserver.annotation.TransformField;
import com.zhili.stationserver.component.stationserver.StationCenter;
import com.zhili.stationserver.configuration.ServerConfig;
import com.zhili.stationserver.dto.*;
import com.zhili.stationserver.util.BytesUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.CuratorFramework;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import static com.zhili.stationserver.constant.TransformType.*;

@Component
@Slf4j
public class KafkaMessageHandler {

    @Autowired
    ServerConfig serverConfig;

    public KafkaMessageHandler(CuratorFramework client) {
    }

    @PostConstruct
    public void init() {
        log.info("KafkaMessageHandler PostConstruct:" + serverConfig);
    }

    /**
     * 启动换电
     *
     * @param record
     * @param ack
     * @throws UnsupportedEncodingException
     */
    @KafkaListener(topics = {"#{serverConfig.getInstanceId()}_CONTROL"}, groupId = "#{serverConfig.getServiceName()}")
    public void processExchangeEvent(ConsumerRecord<String, String> record, Acknowledgment ack) {
        log.info("开始换电数据************:" + record.value());
        BaseControlDTO baseControlDTO = JSON.parseObject(record.value(), BaseControlDTO.class);
        if ("exchangeBegin".equals(baseControlDTO.getType())) {
            StationControlEvent stationControlEvent = JSON.parseObject(baseControlDTO.getData().toString(), StationControlEvent.class);
            StationMessage stationMessage = new StationMessage();
            stationMessage.setCommand(StationMessage.CommandType.START_EXCHANGE);
            StartEventDTO startEventDTO = new StartEventDTO();
            startEventDTO.setCmdNo(stationControlEvent.getCmdNo());
            startEventDTO.setControlSource(stationControlEvent.getControlSource());
            startEventDTO.setMobile(stationControlEvent.getMobile());
            startEventDTO.setDriverId(stationControlEvent.getDriverId());
            startEventDTO.setName(stationControlEvent.getName());
            log.info("启动换电数据************:{}", startEventDTO);
            sendToChannel(stationControlEvent.getStationConnInfo().getConnIdStr(), startEventDTO);
        }
        ack.acknowledge();
    }

    /**
     * 上传换电记录回复
     *
     * @param record
     * @param ack
     */
    @KafkaListener(topics = {"#{serverConfig.getInstanceId()}_UPLOAD_EXCHANGE_RECORD_REPLY"}, groupId = "#{serverConfig.getServiceName()}")
    public void processUploadExchangeReply(ConsumerRecord<String, String> record, Acknowledgment ack) {
        log.info("上传换电记录回复数据************:" + record.value());
        ExchangeUploadReply exchangeUploadReply = JSON.parseObject(record.value(), ExchangeUploadReply.class);
        if (StringUtils.isNotEmpty(exchangeUploadReply.getExchangeNo())) {
            StationMessage stationMessageReply = new StationMessage();
            stationMessageReply.setCommand(StationMessage.CommandType.EXCHANGE_RECORD_REPLY);
            byte[] replyBytes = new byte[33];
            BytesUtil.setSubBytes(replyBytes, 0, BytesUtil.fromString(exchangeUploadReply.getExchangeNo(), 32, (byte) 0));
            replyBytes[32] = (byte) exchangeUploadReply.getResult();
            stationMessageReply.setData(replyBytes);
            StationCenter.sendToChannel(exchangeUploadReply.getConnIdStr(), stationMessageReply);
        }
        ack.acknowledge();
    }


    @KafkaListener(topics = {"#{serverConfig.getInstanceId()}_UPLOAD_CHARGE_RECORD_REPLY"}, groupId = "#{serverConfig.getServiceName()}")
    public void processUploadChargeReply(ConsumerRecord<String, String> record, Acknowledgment ack) {
        //log.info("上传充电记录回复数据************:" + record.value());
        UploadChargeReply uploadChargeReply = JSON.parseObject(record.value(), UploadChargeReply.class);
        if (StringUtils.isNotEmpty(uploadChargeReply.getSerialNo())) {
            StationMessage stationMessageReply = new StationMessage();
            stationMessageReply.setCommand(StationMessage.CommandType.PERCEPTION_CHARGE_RECORD_REPLY);
            byte[] replyBytes = new byte[33];
            BytesUtil.setSubBytes(replyBytes, 0, BytesUtil.fromString(uploadChargeReply.getSerialNo(), 32, (byte) 0));
            replyBytes[32] = (byte) uploadChargeReply.getState();
            stationMessageReply.setData(replyBytes);
            StationCenter.sendToChannel(uploadChargeReply.getConnIdStr(), stationMessageReply);
        }
        ack.acknowledge();
    }


    @KafkaListener(topics = {"#{serverConfig.getInstanceId()}_SET_RATE"}, groupId = "#{serverConfig.getServiceName()}")
    public void cloudIssueRate(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ChargeSetRateDto dto = JSON.parseObject(value, ChargeSetRateDto.class);
        log.info("云端下发费率************:{}",dto);
        CloudIssueRateDto rateDto = new CloudIssueRateDto();
        rateDto.setModelId(dto.getModelId());
        rateDto.setModelName(dto.getModelName());
        rateDto.setLogId(dto.getLogId());
        rateDto.setStopVehRate(dto.getStopVehRate());
        rateDto.setIndexNo(dto.getIndexNo());
        rateDto.setRateDetailList(dto.getRateDetailList());
        sendToChannel(dto.getStationConnInfo().getConnIdStr(), rateDto);
        ack.acknowledge();
    }

    @KafkaListener(topics = {"#{serverConfig.getInstanceId()}_CHARGE"}, groupId = "#{serverConfig.getServiceName()}")
    public void charge(ConsumerRecord<String, String> record, Acknowledgment ack) {
        String value = record.value();
        ChargeDto dto = JSON.parseObject(value, ChargeDto.class);
        log.info("小程序发起充断电：{}",dto);
        if ("startCharge".equals(dto.getType())) {
            ChargeStartDto chargeStartDto = new ChargeStartDto();
            chargeStartDto.setChargerId(dto.getChargerId());
            chargeStartDto.setGunNo(dto.getGunNo());
            chargeStartDto.setUserId(dto.getUserId());
            chargeStartDto.setStrategy(dto.getStrategy());
            chargeStartDto.setStrategyParam(dto.getStrategyParam());
            chargeStartDto.setChgType(dto.getChgType());
            chargeStartDto.setSn(dto.getSn());
            chargeStartDto.setFleetId(dto.getFleetId());
            sendToChannel(dto.getStationConnInfo().getConnIdStr(), chargeStartDto);
        }else if ("stopCharge".equals(dto.getType())) {
            ChargeStopDto chargeStopDto = new ChargeStopDto();
            chargeStopDto.setChargerId(dto.getChargerId());
            chargeStopDto.setGunNo(dto.getGunNo());
            chargeStopDto.setChgType(dto.getChgType());
            sendToChannel(dto.getStationConnInfo().getConnIdStr(), chargeStopDto);
        }
        ack.acknowledge();
    }


    public void sendToChannel(String connIdStr, Object obj) {
        Class cla = obj.getClass();
        //log.info("---------------class name: {}, obj:{}", cla.getName(), JSON.toJSONString(obj));
        ProtocalTranform anno = obj.getClass().getAnnotation(ProtocalTranform.class);
        if (anno == null) {
            return;
        }
        Integer cmd = anno.command();
        Integer size = anno.size();
        StationMessage stationMessage = new StationMessage();
        stationMessage.setCommand(cmd);
        byte[] data = new byte[size];
        Field[] fields = cla.getDeclaredFields();
        for (Field field : fields) {
            try {
                field.setAccessible(true);
                if (field.get(obj) == null) {
                    continue;
                }
                TransformField annoF = field.getAnnotation(TransformField.class);
                if (annoF == null) {
                    continue;
                }
                int start = annoF.start();
                int length = annoF.length();
                String type = annoF.type();
                float accuracy = annoF.accuracy();
                float offset = annoF.offset();
                int segmentLen = annoF.segmentLen();
//                log.info("field name: {}", field.getName());

                if (type.equals(DATE_TYPE)) {
                    Date o = (Date) field.get(obj);
                    long ts = o.getTime();
                    byte[] bytes = BytesUtil.longToBytes(ts);
                    BytesUtil.setSubBytes(data, start, bytes);
                } else if (type.equals(UINT_TYPE)) {
//                    log.info("对象：{},{}", obj.getClass().getName(), JSON.toJSONString(obj));
//                    log.info("转换数据：{},{}", field.getName(), field.get(obj));
//                    byte[] bytes = BytesUtil.fromIntWithLowerFirst(field.getInt(obj), length);
                    Object o = field.get(obj);
                    byte[] bytes = BytesUtil.fromIntWithLowerFirst(Integer.valueOf(o.toString()), length);
                    BytesUtil.setSubBytes(data, start, bytes);
                } else if (type.equals(STRING_TYPE)) {
                    byte[] bytes = field.get(obj).toString().getBytes("GBK");
                    BytesUtil.setSubBytes(data, start, bytes);
                } else if (type.equals(TIME_STRING_TYPE)) {
                    String s = field.get(obj).toString();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Date parse = sdf.parse(s);
                    byte[] bytes = BytesUtil.composeTimeBytes(parse);
                    BytesUtil.setSubBytes(data, start, bytes);
                } else if (type.equals(FLOAT_TYPE)) {
                    Object o = field.get(obj);
                    if (o != null) {
                        byte[] bytes = BytesUtil.fromIntWithLowerFirst((int) ((Float.valueOf(o.toString()) - offset) / accuracy), length);
                        BytesUtil.setSubBytes(data, start, bytes);
                    } else {
                        byte[] bytes = BytesUtil.fromIntWithLowerFirst((int) ((annoF.floatDefault() - offset) / accuracy), length);
                        BytesUtil.setSubBytes(data, start, bytes);
                    }
                } else if (type.equals(BYTES_TYPE)) {
                    Object o = field.get(obj);
                    byte[] bytes = (byte[]) o;
                    BytesUtil.setSubBytes(data, start, bytes);
                } else if (type.equals(RATE_LIST_TYPE)) {
                    Object o = field.get(obj);
                    List<RateDetailDto> list = (List<RateDetailDto>) o;
                    byte[] bytes = new byte[length];
                    for (int i = 0; i < list.size(); i++) {
                        BytesUtil.setSubBytes(bytes, i * segmentLen, BytesUtil.fromIntWithLowerFirst(list.get(i).getStartHour(), 1));
                        BytesUtil.setSubBytes(bytes, i * segmentLen + 1, BytesUtil.fromIntWithLowerFirst(list.get(i).getStartMinute(), 1));
                        BytesUtil.setSubBytes(bytes, i * segmentLen + 2, BytesUtil.fromIntWithLowerFirst(list.get(i).getRateType(), 1));
                        BytesUtil.setSubBytes(bytes, i * segmentLen + 3, BytesUtil.fromIntWithLowerFirst(Math.round(list.get(i).getChargeRate() * 1000000), 4));
                        BytesUtil.setSubBytes(bytes, i * segmentLen + 7, BytesUtil.fromIntWithLowerFirst(Math.round(list.get(i).getSerRate() * 1000000), 4));
                    }
                    BytesUtil.setSubBytes(data, start, bytes);
                }
            } catch (Exception e) {
                e.printStackTrace();
                continue;
            }
        }
        stationMessage.setData(data);
        //log.info("data:{}", ArrayUtils.toString(data));
        StationCenter.sendToChannel(connIdStr, stationMessage);
    }
}
